@@ -2,7 +2,7 @@
 * config_names.h
 
 
-Copyright (C) 2008 Alessandro Vesely
+Copyright (C) 2008-2010 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -23,11 +23,11 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef CONFIG_NAMES_H_INCLUDED
 #define CONFIG_NAMES_H_INCLUDED
 
-#if defined HAVE_CONFIG
+//#if defined HAVE_CONFIG
 #include <config.h>
-#else
-#define PACKAGE_VERSION "unknown"
-#endif
+//#else
+//#define PACKAGE_VERSION "unknown"
+//#endif
 
 /*
 * These are "reasonable" configuration values that can be overridden
@@ -39,7 +39,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 // a basename prefix.
 #if !defined IPQBDB_DATABASE_PREFIX
 #define IPQBDB_DATABASE_PREFIX "/var/lib/ipqbdb/"
-//#define IPQBDB_DATABASE_PREFIX "/home/ale/tmp/ipqbdbd/"
 #endif
 
 // The database suffix will be used unless the name supplied already ends
@@ -131,8 +130,13 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 // This value should not be used as a flag to have static records.
 // (records that have this decay value are deleted jut like the other ones.)
 #if !defined IPQBDB_NEVER_DECAY
-#define IPQBDB_NEVER_DECAY 50457600400
+#define IPQBDB_NEVER_DECAY 50457600400.0
 #endif
 
+// Define IPQBDB_DEBUG to turn on assertion and miscellaneous (annoying)
+// printing. This macro is only used here, the code uses NDEBUG.
+#if ! defined IPQBDB_DEBUG
+#define NDEBUG
+#endif
 
 #endif // CONFIG_NAMES_H_INCLUDED
