@@ -343,13 +343,6 @@ int main(int argc, char const *argv[])
 	}
 	else
 	{
-		if (poptPeekArg(opt) != NULL)
-		{
-			fprintf(stderr, "%s: unexpected argument: %s\n",
-				err_prefix, poptGetArg(opt));
-			errs = 1;
-		}
-
 		if (version_opt)
 		{
 			fprintf(stdout, "%s: version " PACKAGE_VERSION "\n", err_prefix);
@@ -367,16 +360,6 @@ int main(int argc, char const *argv[])
 			errs = 2;
 		}
 
-		if (get_ip_address(&ip_addr))
-			errs = 3;
-
-		if (initial_decay < 0.0)
-		{
-			fprintf(stdout, "%s: initial decay %g is negative\n",
-				err_prefix, initial_decay);
-			errs = 3;
-		}
-
 		// popt sets verbose to 0 if no arg is given
 		verbose += 1;
 
@@ -387,9 +370,26 @@ int main(int argc, char const *argv[])
 			openlog(err_prefix, LOG_PID, LOG_DAEMON);
 			ap.mode = LOG_DAEMON;
 		}
+
+		if (poptPeekArg(opt) != NULL)
+		{
+			report_error(&ap, LOG_ERR, "unexpected argument: %s\n",
+				poptGetArg(opt));
+			errs = 1;
+		}
+
+		if (get_ip_address(&ip_addr))
+			errs = 3;
+
+		if (initial_decay < 0.0)
+		{
+			report_error(&ap, LOG_WARNING, "initial decay %g is negative\n",
+				initial_decay);
+			errs = 3;
+		}
 	}
 
-	if (errs == 1)
+	if (errs == 1 && !syslog_opt)
 		poptPrintUsage(opt, stdout, 0);
 	poptFreeContext(opt);
 	rtc = 0;
