@@ -399,10 +399,11 @@ static char *min_time = "0";
 static char *max_probability = "100%";
 static char *min_probability = "0%";
 static char *reason_string = "";
+static char *max_records_opt = "0";
+static long max_records = 0;
 static int version_opt, help_opt, syslog_opt, cleanup_opt;
 static int only_blocked, non_blocked;
 static int verbose = -1;
-static int max_records_opt = 0;
 static int list_opt, raw_opt, del_opt, ban_opt, stat_opt;
 static struct poptOption opttab[] =
 {
@@ -420,8 +421,8 @@ static struct poptOption opttab[] =
 	"Select records with probability higher than given(*)", "int[%]"},
 	{"max-probability", '\0', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_probability, 0,
 	"Select records with probability lower than given(*)", "int[%]"},
-	{"max-records", 'M', POPT_ARG_INT|POPT_ARGFLAG_SHOW_DEFAULT, &max_records_opt, 0,
-	"Limit selection to older records that exceed the given number", "int"},
+	{"max-records", 'M', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_records_opt, 0,
+	"Limit selection to older records that exceed the given number", "int[K|M|G]"},
 	{"ip-addr", 'i', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &ip_address, 0,
 	"The single address or range to delete or list, if any", "ip[-last|/cidr]"},
 	{"reason", 'r', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &reason_string, 0,
@@ -434,7 +435,7 @@ static struct poptOption opttab[] =
 	"List selected records displaying plain numbers", NULL},
 	{"del", '\0', POPT_ARG_NONE, &del_opt, 0,
 	"Delete selected records", NULL},
-	{"stats", '\0', POPT_ARG_NONE, &stat_opt, 0,
+	{"stats", 'S', POPT_ARG_NONE, &stat_opt, 0,
 	"List by descr stat counters", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
 	"Be verbose", "level"},
@@ -626,6 +627,38 @@ static int get_seconds(time_t *secs)
 	return 0;
 }
 
+static int get_max_records(void)
+{
+	errno = 0;
+	char *t = NULL;
+	unsigned long long l = strtoull(max_records_opt, &t, 10);
+	if (t && *t && errno == 0)
+	{
+		switch (toupper(*(unsigned char*)t))
+		{
+			case 'G':
+				l *= 1024ULL;
+				// all through...
+			case 'M':
+				l *= 1024ULL;
+			case 'K':
+				l *= 1024ULL;
+				++t;
+			default:
+				break;
+		}
+	}
+
+	if (t == NULL || *t != 0 || l > LONG_MAX)
+	{
+		report_error(&ap, LOG_ERR, "invalid max-records %s\n", max_records_opt);
+		return 1;
+	}
+
+	max_records = (long)l;
+	return 0;
+}
+
 static int get_probability(int *prob, char *probability)
 {
 	char *t = NULL;
@@ -1062,7 +1095,7 @@ int main(int argc, char const *argv[])
 			if (raw_opt)
 				list_opt = 1;
 
-			if (!list_opt && !del_opt && !ban_opt && !stat_opt)
+			if (!list_opt && !del_opt && !ban_opt && !stat_opt && errs == 0)
 			{
 				fprintf(stdout,
 					"%s: at least one of"
@@ -1083,6 +1116,7 @@ int main(int argc, char const *argv[])
 		}
 
 		if (get_ip_address(&ip) || get_seconds(&secs) ||
+			get_max_records() ||
 			get_probability(&prob[0], min_probability) ||
 			get_probability(&prob[1], max_probability))
 				errs = 3;
@@ -1134,7 +1168,7 @@ int main(int argc, char const *argv[])
 			if (rtc == 0 && caught_signal == 0)
 				rtc = all_reason_id(db_de1, db_de2);
 
-			if (rtc == 0 && caught_signal == 0 && max_records_opt > 0)
+			if (rtc == 0 && caught_signal == 0 && max_records > 0)
 			{
 				histog his;
 				if (init_histog(&his, secs) == 0)
@@ -1142,19 +1176,19 @@ int main(int argc, char const *argv[])
 					rtc = do_del(db_block, db_de1, &ip, secs, prob, &his);
 					if (rtc == 0)
 					{
-						if (his.total_recs <= max_records_opt)
+						if (his.total_recs <= max_records)
 						{
 							if (verbose)
 								printf(
-									"total records = %d <= %d: no deletion needed!\n",
-									his.total_recs, max_records_opt);
+									"total records = %d <= %ld: no deletion needed!\n",
+									his.total_recs, max_records);
 							del_opt = 0;
 						}
 						else
-							if (his.total_recs - his.total_selected < max_records_opt)
+							if (his.total_recs - his.total_selected < max_records)
 						{
 							char const* err =
-								increase_secs(&his, max_records_opt, secs);
+								increase_secs(&his, max_records, secs);
 							if (err)
 								report_error(&ap, LOG_ERR,
 									"increase_secs: %s\n", err);
@@ -1189,10 +1223,10 @@ int main(int argc, char const *argv[])
 						{
 							if (verbose)
 								printf("total records = %d, selected = %d,"
-									" expected = %d >= %d: no adjustment possible!\n",
+									" expected = %d >= %ld: no adjustment possible!\n",
 									his.total_recs, his.total_selected,
 									his.total_recs - his.total_selected,
-										max_records_opt);
+										max_records);
 						}
 					}
 					free(his.his);
