@@ -3,7 +3,7 @@
 * written by ale in milano on 28 sep 2008
 * read from a socket and extract pcre patterns to add to block.db
 
-Copyright (C) 2008 Alessandro Vesely
+Copyright (C) 2008-2010 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -65,9 +65,8 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <assert.h>
 
-static char *trim(char *s)
+static char *trim(char *s, char *end)
 {
-	char *end = s + strlen(s);
 	while (s < end && isspace(*(unsigned char*)s))
 		++s;
 	while (s < end && isspace(*(unsigned char*)--end))
@@ -115,7 +114,7 @@ static char *next_field(char **ps)
 }
 
 static app_private ap;
-static char const err_prefix[] = "ipqbdbd";
+static char const err_prefix[] = "ibd-parse";
 int caught_signal;
 
 #if !defined NDEBUG
@@ -183,6 +182,7 @@ static char *db_white_name = IPQBDB_WHITE_DATABASE_NAME;
 static char *db_descr_name = IPQBDB_DESCR_DATABASE_NAME;
 static char *pcre_name = IPQBDB_PCRE_FILE;
 static int version_opt, help_opt, no_cleanup_opt, chartable_opt, no_daemon_opt;
+static int exit_on_eof_opt, no_run_opt;
 static int verbose = -1;
 static struct poptOption opttab[] =
 {
@@ -193,15 +193,19 @@ static struct poptOption opttab[] =
 	{"db-descr", 'd', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_descr_name, 0,
 	"The reason description table.", "filename"},
 	{"fifo-socket", 's', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &socket_name, 0,
-	"The listening mkfifo socket.", "name"},
+	"The listening fifo socket.", "name"},
 	{"pcre-file", 'f', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &pcre_name, 0,
 	"The pcre action file", "fullpath"},
 	{"initial-decay", 't', POPT_ARG_DOUBLE|POPT_ARGFLAG_SHOW_DEFAULT, &initial_decay, 0,
 	"The default time for the block probability to halve (seconds)", "float"},
 	{"initial-count", 'c', POPT_ARG_INT|POPT_ARGFLAG_SHOW_DEFAULT, &initial_count, 0,
 	"Default initial full probability count", "integer"},
+	{"exit-on-eof", '\0', POPT_ARG_NONE, &exit_on_eof_opt, 0,
+	"Exit when EOF is found on the fifo socket", NULL},
 	{"no-daemon", 'D', POPT_ARG_NONE, &no_daemon_opt, 0,
 	"Stay foreground and use stderr", NULL},
+	{"no-run", 'n', POPT_ARG_NONE, &no_run_opt, 0,
+	"Only check options and pcre action file", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
 	"Be verbose", "level"},
 	{"pcre-use-locale", 'l', POPT_ARG_NONE, &chartable_opt, 0,
@@ -231,20 +235,21 @@ typedef struct re_list_t
 
 typedef struct initialization_stuff
 {
-	int inited;
-
 	DB_ENV* db_env;
 	DB *db1, *db2;
 
 	re_list_t *re_list;
 	int max_sub;
 
-	char *home_dir;
-	int default_ini_prob;
+	int inited;
 
+	char *home_dir;
 	char **db_fnames;
 	size_t fnames_alloc, fnames_count;
 
+	int line;
+
+	int default_ini_prob;
 	int verbose_header_given;
 } initialization_stuff;
 
@@ -253,6 +258,7 @@ typedef struct runtime_stuff
 	char *home_dir;
 
 	DB_ENV *db_env;
+	DB *db1, *db2;
 	DB **db, *db_white;
 
 	re_list_t *re_list;
@@ -278,18 +284,9 @@ typedef struct socket_stuff
 
 static unsigned char const *pcre_chartables = NULL;
 
-static void clear_run(runtime_stuff *run)
+static void clear_re_list(re_list_t *re_list)
 {
-	if (run->db)
-		for (size_t i = 0; i < run->db_count; ++i)
-			close_db(run->db[i]);
-
-	close_db(run->db_white);
-	close_dbenv(run->db_env, !no_cleanup_opt);
-	free(run->db);
-	free(run->home_dir);
-
-	for (re_list_t *r = run->re_list; r != NULL;)
+	for (re_list_t *r = re_list; r != NULL;)
 	{
 		void *tmp = r;
 		pcre_free(r->code);
@@ -297,7 +294,21 @@ static void clear_run(runtime_stuff *run)
 		r = r->next;
 		free(tmp);
 	}
+}
 
+static void clear_run(runtime_stuff *run)
+{
+	if (run->db)
+		for (size_t i = 0; i < run->db_count; ++i)
+			close_db(run->db[i]);
+
+	close_db(run->db_white);
+	close_db(run->db1);
+	close_db(run->db2);
+	close_dbenv(run->db_env, !no_cleanup_opt);
+	free(run->db);
+	free(run->home_dir);
+	clear_re_list(run->re_list);
 	memset(run, 0, sizeof *run);
 }
 
@@ -315,9 +326,49 @@ static void clear_sock(socket_stuff *sock)
 	sock->fd = -1;
 }
 
+static char*
+my_database_fname(initialization_stuff *ini, char const *name)
+{
+	char *fname = database_fname_with_home(name, &ap, ini->home_dir, 1);
+	if (fname)
+	{
+		char *p = strrchr(fname, '/');
+		if (p == NULL || p == fname)
+		{
+			report_error(&ap, LOG_ERR, "invalid name \"%s\" at line %d in %s\n",
+				fname, ini->line, pcre_name);
+			free(fname);
+			return NULL;
+		}
+
+		*p = 0;
+		if (ini->home_dir)
+		{
+			// let all db's share an env in their home directory
+			if (strcmp(ini->home_dir, fname) != 0)
+			{
+				*p = '/';
+				report_error(&ap, LOG_ERR,
+					"%s has a directory different than %s at line %d in %s\n",
+					fname, ini->home_dir, ini->line, pcre_name);
+				free(fname);
+				return NULL;
+			}
+		}
+		else if ((ini->home_dir = strdup(fname)) == NULL)
+		{
+			free(fname);
+			return NULL;
+		}
+		*p = '/';
+	}
+	
+	return fname;
+}
+
 static void init_inidb(initialization_stuff *ini)
 {
-	char *descr_fname = database_fname(db_descr_name, &ap);
+	char *descr_fname = my_database_fname(ini, db_descr_name);
 
 	if (descr_fname &&
 		open_database(descr_fname, &ap, &ini->db_env, NULL) == 0 &&
@@ -329,7 +380,7 @@ static void init_inidb(initialization_stuff *ini)
 
 static int add_fname(initialization_stuff *ini, char *fname)
 {
-	char *dir_name = database_fname(fname, &ap);
+	char *dir_name = my_database_fname(ini, fname);
 	if (dir_name)
 	{
 		size_t i;
@@ -340,18 +391,6 @@ static int add_fname(initialization_stuff *ini, char *fname)
 			return -1;
 
 		*p = 0;
-		if (ini->home_dir)
-		{
-			// simpler if all db's share an env in their home directory
-			if (strcmp(ini->home_dir, dir_name) != 0)
-			{
-				free(dir_name);
-				return -4;
-			}
-		}
-		else if ((ini->home_dir = strdup(dir_name)) == NULL)
-			return -1;
-
 		for (i = 0; i < ini->fnames_count; ++i)
 			if (strcmp(db_fname, ini->db_fnames[i]) == 0)
 			{
@@ -383,11 +422,12 @@ static int add_fname(initialization_stuff *ini, char *fname)
 			free(dir_name);
 			return i;
 		}
+		free(dir_name);
 	}
 	return -1;
 }
 
-static int insert_pcre(initialization_stuff *ini, int line,
+static int insert_pcre(initialization_stuff *ini,
 	char *re, char *ip, char *reason, char *count, char *fname, char *decay)
 {
 	int errs = 0, rc;
@@ -400,7 +440,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 
 	memset(&r, 0, sizeof r);
 
-	r.line = line;
+	r.line = ini->line;
 
 	/*
 	* The symbol '*' in ip indicates to replace <HOST>, before compiling...
@@ -414,7 +454,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 		{
 			report_error(&ap, LOG_ERR,
 				"replacement \"%s\" is missing or ambiguous at line %d in %s\n",
-				host_repl, line, pcre_name);
+				host_repl, ini->line, pcre_name);
 			errs |= 8;
 		}
 		else
@@ -439,7 +479,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 	{
 		report_error(&ap, LOG_ERR,
 			"%s \"%.*s<< ABOUT HERE>>%s\" at line %d in %s\n",
-			error, err_off, re, re + err_off, line, pcre_name);
+			error, err_off, re, re + err_off, ini->line, pcre_name);
 		errs |= 2;
 		tot_sub = INT_MAX;
 	}
@@ -450,7 +490,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 		{
 			report_error(&ap, LOG_WARNING,
 				"expression reported as %s at line %d in %s\n",
-				error, line, pcre_name);
+				error, ini->line, pcre_name);
 			errs |= 1;
 		}
 
@@ -494,7 +534,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 				}
 				printf("%10d %10.10s %10d %10zu %10zu %10d %s\n",
 					backrefmax, firststr, lastliteral, size,
-					studysize, line, re);
+					studysize, ini->line, re);
 			}
 			else
 			{
@@ -534,7 +574,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 		{
 			report_error(&ap, LOG_ERR,
 				"missing (?P<IP>...) named sub expression at line %d in %s\n",
-				line, pcre_name);
+				ini->line, pcre_name);
 			errs |= 2;
 		}
 		else if (count == 1 && ndx > 0 && ndx < tot_sub)
@@ -546,7 +586,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 	{
 		report_error(&ap, LOG_ERR,
 			"invalid ip sub num \"%s\" (pattern has %d subs) at line %d in %s\n",
-			ip, tot_sub, line, pcre_name);
+			ip, tot_sub, ini->line, pcre_name);
 		errs |= 2;
 	}
 
@@ -581,8 +621,10 @@ static int insert_pcre(initialization_stuff *ini, int line,
 		{
 			r.init_prob = 1;
 			report_error(&ap, *t == 0? LOG_NOTICE: LOG_ERR,
-				"%s is too high: initial probability set to 1 at line %d in %s\n",
-				count, line, pcre_name);
+				"initial count %s is %s: %s at line %d in %s\n",
+				count, *t == 0? "too hight": "bad",
+				*t == 0? "initial probability set to 1": "error",
+				ini->line, pcre_name);
 			if (*t)
 				errs |= 2;
 		}
@@ -596,10 +638,6 @@ static int insert_pcre(initialization_stuff *ini, int line,
 	r.db_ndx = add_fname(ini, fname? fname: db_block_name);
 	if (r.db_ndx < 0)
 	{
-		if (r.db_ndx == -4)
-			report_error(&ap, LOG_ERR,
-				"%s has a directory different than %s at line %d in %s\n",
-				fname? fname: db_block_name, ini->home_dir, line, pcre_name);
 		errs |= 2;
 	}
 
@@ -612,7 +650,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 		if (*t != 0 || !isnormal(r.decay) || r.decay <= 0)
 		{
 			report_error(&ap, LOG_ERR, "invalid decay %s at line %d in %s\n",
-				decay, line, pcre_name);
+				decay, ini->line, pcre_name);
 			errs |= 2;
 		}
 	}
@@ -644,7 +682,7 @@ static int insert_pcre(initialization_stuff *ini, int line,
 	return errs;
 }
 
-static int parse_pcre(char *s, int line, initialization_stuff *ini)
+static int parse_pcre(char *s, initialization_stuff *ini)
 {
 	char *re, *ip, *reason, *count = NULL, *fname = NULL, *decay = NULL;
 	int delim = s[0];
@@ -657,7 +695,7 @@ static int parse_pcre(char *s, int line, initialization_stuff *ini)
 	{
 		report_error(&ap, LOG_ERR,
 			"missing matching %c RE delimiter at line %d in %s\n",
-			delim, line, pcre_name);
+			delim, ini->line, pcre_name);
 		return 1;
 	}
 
@@ -667,7 +705,7 @@ static int parse_pcre(char *s, int line, initialization_stuff *ini)
 	{
 		report_error(&ap, LOG_ERR,
 			"missing ip subexpression number at line %d in %s\n",
-			line, pcre_name);
+			ini->line, pcre_name);
 		return 1;
 	}
 
@@ -676,7 +714,7 @@ static int parse_pcre(char *s, int line, initialization_stuff *ini)
 	{
 		report_error(&ap, LOG_ERR,
 			"missing reason field at line %d in %s\n",
-			line, pcre_name);
+			ini->line, pcre_name);
 		return 1;
 	}
 
@@ -693,7 +731,7 @@ static int parse_pcre(char *s, int line, initialization_stuff *ini)
 				if (x)
 					report_error(&ap, LOG_WARNING,
 						"ignoring extra stuff \"%s\" at line %d in %s\n",
-						x, line, pcre_name);
+						x, ini->line, pcre_name);
 				if (decay[0] == '-' && decay[1] == 0)
 					decay = NULL;
 			}
@@ -704,10 +742,23 @@ static int parse_pcre(char *s, int line, initialization_stuff *ini)
 			count = NULL;
 	}
 
-	return insert_pcre(ini, line, re, ip, reason, count, fname, decay);
+	return insert_pcre(ini, re, ip, reason, count, fname, decay);
 }
 
-enum { read_pcre_after_sighup = 0, read_pcre_daemonize_if_ok};
+static void ini_error_clear(initialization_stuff *ini)
+{
+	if (ini->inited)
+	{
+		close_db(ini->db1);
+		close_db(ini->db2);
+		close_dbenv(ini->db_env, !no_cleanup_opt);
+	}
+	free(ini->db_fnames);
+	free(ini->home_dir);
+	clear_re_list(ini->re_list);
+}
+
+enum { read_pcre_and_thats_it = 0, read_pcre_daemonize_if_ok};
 static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 /*
 * run is cleared before entry. This function initializes it.
@@ -715,13 +766,13 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 */
 {
 	char buf[4096], *s;
-	int line = 0;
 	int errs = 0;
 	FILE *fp = fopen(pcre_name, "r");
 
 	initialization_stuff ini;
 	memset(&ini, 0, sizeof ini);
 
+	// set initial probability from prog args
 	if (!((unsigned)initial_count < CHAR_BIT * sizeof(int) &&
 		(ini.default_ini_prob = RAND_MAX >> initial_count) > 0))
 			ini.default_ini_prob = 1;
@@ -733,15 +784,29 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 		return -1;
 	}
 
+	// read config
 	while ((s = fgets(buf, sizeof buf, fp)) != NULL)
-		errs |= parse_pcre(trim(s), ++line, &ini);
+	{
+		size_t len = strlen(s);
+		if (len + 1 >= sizeof buf && s[len-1] != '\n')
+		{
+			unsigned extra = 1;
+			int ch = fgetc(fp);
+			while (ch != '\n' && ch != EOF)
+			{
+				ch = fgetc(fp);
+				++extra;
+			}
+			report_error(&ap, LOG_ERR,
+				"line %d is %u chars too long, in %s\n",
+					ini.line + 1, extra, pcre_name);
+			errs |= 1;
+		}
+		ini.line += 1;
+		errs |= parse_pcre(trim(s, s + len), &ini);
+	}
 
 	fclose(fp);
-	if (ini.inited)
-	{
-		close_db(ini.db1);
-		close_db(ini.db2);
-	}
 
 	if (errs ||
 		ini.fnames_count <= 0 ||
@@ -752,7 +817,7 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 			"Failed to initialize: %s\n",
 			errs? "errors in pcre file":
 					"no block db referenced");
-		close_dbenv(ini.db_env, !no_cleanup_opt);
+		ini_error_clear(&ini);
 		return -1;
 	}
 
@@ -775,7 +840,7 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 		{
 			report_error(&ap, LOG_CRIT, "cannot %s: %s - exiting\n",
 				what, strerror(errno));
-			close_dbenv(ini.db_env, !no_cleanup_opt);
+			ini_error_clear(&ini);
 			return -1;
 		}
 	}
@@ -783,13 +848,15 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 	{
 		report_error(&ap, LOG_CRIT, "cannot chdir to %s: %s\n",
 			ini.home_dir, strerror(errno));
-		close_dbenv(ini.db_env, !no_cleanup_opt);
+		ini_error_clear(&ini);
 		return -1;
 	}
 
 	run->home_dir = ini.home_dir;
 	run->db_count = ini.fnames_count;
 	run->db_env = ini.db_env;
+	run->db1 = ini.db1;
+	run->db2 = ini.db2;
 	run->db = (DB**)malloc(ini.fnames_count * sizeof(DB*));
 	run->ovecsize = (ini.max_sub + 1) * 3;
 	run->ovector = (int*)malloc(run->ovecsize * sizeof(int));
@@ -833,6 +900,36 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 	return errs;
 }
 
+static int try_read_pcre(runtime_stuff *run)
+/*
+* only change run if read_pcre worked ok
+* if the runtime stuff is not NULL, it has already been initialized by
+* a previous call to read_pcre, and one live environment will remain.
+*/
+{
+	runtime_stuff newrun;
+	memset(&newrun, 0, sizeof newrun);
+
+	int const save_no_cleanup_opt = no_cleanup_opt;
+	if (run)
+		no_cleanup_opt = 1;
+
+	if (chartable_opt && pcre_chartables == NULL)
+		pcre_chartables = pcre_maketables();
+
+	int const rtc = read_pcre(read_pcre_and_thats_it, &newrun);
+	if (rtc == 0 && run)
+	{
+		clear_run(run);
+		*run = newrun;
+	}
+	else
+		clear_run(&newrun);
+
+	no_cleanup_opt = save_no_cleanup_opt;
+	return rtc;
+}
+
 static int get_ip_address(uint32_t* ip, char *dest, size_t dsize,
 	char const *src, size_t ssize, int ex_line)
 {
@@ -964,7 +1061,7 @@ static int block_matched(runtime_stuff *run, re_list_t const * const r,
 		{
 			rtc = write_block(run->db[r->db_ndx], ip_addr, decay,
 				r->reason, r->init_prob, 0,
-				verbose? display_written_block: NULL, NULL);
+				verbose? display_written_block: NULL, run->db1);
 
 			if (rtc < 0)
 				err = 1;
@@ -979,8 +1076,12 @@ static int run_pcre(runtime_stuff *run, char const *str, size_t const len)
 * run all expression until one is found.
 * if a match is found, set the record in the corresponding block db.
 * return -1 if an unexpected error occurs.
+*
+* NB: this function assumes that the subject string, str, terminates with
+* a \n char; thus no\n is added to error messages that include it.
 */
 {
+	int rtc = 0;
 	re_list_t *r = run->re_list, *prev = NULL;
 	while (r)
 	{
@@ -990,9 +1091,12 @@ static int run_pcre(runtime_stuff *run, char const *str, size_t const len)
 		if (rc < 0) // failed
 		{
 			if (rc != PCRE_ERROR_NOMATCH)
+			{
 				report_error(&ap, LOG_WARNING,
 					"pcre_exec returned %d for expr at line %d, subject=%s"/* no\n */,
 					rc, r->line, str);
+				rtc = -1;
+			}
 			prev = r;
 			r = r->next;
 		}
@@ -1002,7 +1106,10 @@ static int run_pcre(runtime_stuff *run, char const *str, size_t const len)
 
 			++r->match;
 			if (err)
+			{
 				++r->errs;
+				rtc = -1;
+			}
 			/*
 			* move matched rule to front, if not already there
 			*/
@@ -1019,21 +1126,21 @@ static int run_pcre(runtime_stuff *run, char const *str, size_t const len)
 	if (r == NULL && verbose > 1)
 		report_error(&ap, LOG_DEBUG, "no match for %s"/* no\n */, str);
 
-	return 0;
+	return rtc;
 }
 
 static void log_re_list(runtime_stuff *run)
 {
 	re_list_t *r = run->re_list;
 	report_error(&ap, LOG_INFO,
-		"%5.5s %5.5s %5.5s %5.5s %5.5s %19.19s %5.5s %5.5s\n",
+		"%5.5s %5.5s %5.5s %5.5s %5.5s %19.19s %6.6s %5.5s\n",
 		"LINE", "MATCH", "ERRS", "subex",
-		"db", "ini. block prob.", "Rcode", "decay");
+		"db", "initial probability", "reason", "decay");
 	while (r)
 	{
 		report_error(&ap, LOG_INFO,
 			"%5d %5u %5u %5d %5d"
-			" %10d %6.2f%% %5d %g\n",
+			" %11d %6.2f%% %6d %5g\n",
 			r->line, r->match, r->errs, r->ip_sub, r->db_ndx,
 			r->init_prob, percent_prob(r->init_prob), r->reason, r->decay);
 		r = r->next;
@@ -1167,7 +1274,11 @@ static int read_socket(socket_stuff *sock)
 			sock->in = sock->len = 0;
 			close(sock->fd);
 			sock->fd = -1;
-			report_error(&ap, LOG_NOTICE, "found EOF on %s\n", sock->name);
+			
+			char *fname = sock->name;
+			if (no_daemon_opt && exit_on_eof_opt)
+				fname = socket_name;
+			report_error(&ap, LOG_NOTICE, "found EOF on %s\n", fname);
 			return 0;
 		}
 	}
@@ -1252,6 +1363,14 @@ int main(int argc, char const *argv[])
 				err_prefix, initial_decay);
 			errs = 2;
 		}
+		
+		if (no_run_opt)
+		{
+			if (try_read_pcre(NULL))
+				errs = 3; // bad
+			else if (errs == 0)
+				errs = 5; // good
+		}
 	}
 
 	if (errs == 1)
@@ -1261,7 +1380,8 @@ int main(int argc, char const *argv[])
 
 	if (errs)
 	{
-		rtc = 1;
+		// status 0 for automatic configuration checking
+		rtc = errs == 5? 0: 1;
 	}
 	else
 	{
@@ -1280,9 +1400,14 @@ int main(int argc, char const *argv[])
 			while (rtc == 0)
 			{
 				rtc = read_socket(&sock);
-				if (rtc == 0 && sock.len > 0)
+				if (rtc == 0)
 				{
-					if (run_pcre(&run, sock.buf, sock.len) < 0)
+					if (sock.len > 0)
+					{
+						// useless to exit on failure: keep reading
+						/* rtc = */ run_pcre(&run, sock.buf, sock.len);
+					}
+					else if (sock.fd == -1 /* EOF */ && exit_on_eof_opt)
 						break;
 				}
 				else if (rtc > 0) // interrupted by signal or reopen socket
@@ -1296,8 +1421,12 @@ int main(int argc, char const *argv[])
 							break;
 
 						case SIGHUP:
-							clear_run(&run);
-							rtc = read_pcre(read_pcre_after_sighup, &run);
+							if (try_read_pcre(&run))
+								report_error(&ap, LOG_CRIT,
+									"errors reading %s after %s:"
+									" keeping existing configuration\n",
+									pcre_name, strsignal(SIGHUP));
+							rtc = 0;
 							break;
 
 						case SIGUSR1:
@@ -1320,7 +1449,6 @@ int main(int argc, char const *argv[])
 					caught_signal = 0;
 				}
 			}
-			rtc -= 1;
 			clear_sock(&sock);
 		}
 		clear_run(&run);
